﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Emotiv;
using Microsoft.Azure.Devices.Client;
using Newtonsoft.Json;
using static System.Console;

namespace BrainComputerInterface
{
    class Program
    {
        static bool stopHeadsetInformation = true;  //GetHeadsetInformation()
        static int userID = -1;  //GetBrainInterfacePower()
        static DeviceClient deviceClient;
        static string _iotHubUri = "ADD YOUR IOT NAME.azure-devices.net";
        static string _deviceKey = "ADD DEVICE KEY";
        static int _ActivityId = 0;
        static int _DeviceId = 1;
        static string _UserName = "BrainComputerInterface001";

        static void Main(string[] args)
        {
            WriteLine("You are uploading to: " + _iotHubUri);
            WriteLine("Current username is:  " + _UserName + "with device id of: " + _DeviceId.ToString());
            WriteLine();
            WriteLine(" 1  - *Baseline");
            WriteLine(" 3  - Writing");
            WriteLine(" 4  - *Reading");
            WriteLine(" 7  - Relaxation");
            WriteLine(" 9  - *Watching media");
            WriteLine(" 10 - Public Speaking");
            WriteLine(" 11 - *Listening to Music");
            WriteLine(" 13 - *Working at Computer");
            WriteLine(" 14 - Programming");
            WriteLine(" 16 - Meeting");
            WriteLine(" 17 - Exercising");
            WriteLine(" 19 - *Meditation");
            WriteLine(" XX - *Playing Music(20)*");
            WriteLine(" 21 - Visualize Push");
            WriteLine(" 22 - Visualize Pull");
            WriteLine();
            Write("Enter what you will be doing while uploading the brain waves: ");

            try
            {
                _ActivityId = Convert.ToInt32(ReadLine());
                WriteLine("You selected: " + _ActivityId.ToString());

                WriteLine("Hit any key to begin uploading your brain waves");
                ReadLine();
                WriteLine("Session Starting...");

                deviceClient = DeviceClient.Create(_iotHubUri, new DeviceAuthenticationWithRegistrySymmetricKey(_UserName, _deviceKey));
                GetHeadsetInformation();
                GetBrainInterfaceMeasurements();
                ReadLine();
            }
            catch (Exception ex)
            {
                WriteLine("Please make sure you entered the number.");
                WriteLine(ex.Message);
                ReadLine();
            }
        }
        static void GetHeadsetInformation()
        {
            EmoEngine engine = EmoEngine.Instance;
            engine.EmoStateUpdated += new EmoEngine.EmoStateUpdatedEventHandler(engine_EmoStateUpdated);
            engine.Connect();

            while (stopHeadsetInformation)
            {
                if (KeyAvailable) break;
                engine.ProcessEvents();
                if ((int)userID == -1)
                    continue;
            }
            engine.Disconnect();
            BackgroundColor = ConsoleColor.Black;
            WriteLine("I am completly operational and all systems are go");
            engine.EmoStateUpdated -= new EmoEngine.EmoStateUpdatedEventHandler(engine_EmoStateUpdated);
        }
        static void engine_EmoStateUpdated(object sender, EmoStateUpdatedEventArgs e)
        {
            EmoState es = e.emoState;
            if (e.userId != 0) return;

            float timeFromStart = es.GetTimeFromStart();
            EdkDll.IEE_SignalStrength_t signalStrength = es.GetWirelessSignalStatus();
            Int32 maxChargeLevel = 0;
            Int32 chargeLevel = 0;
            es.GetBatteryChargeLevel(out chargeLevel, out maxChargeLevel);

            if (chargeLevel > 0 && signalStrength > EdkDll.IEE_SignalStrength_t.BAD_SIG)
            {
                stopHeadsetInformation = false;
                BackgroundColor = ConsoleColor.DarkGreen;
            }

            WriteLine("Time: " + timeFromStart);
            WriteLine("Signal Strength: " + signalStrength);
            WriteLine("Charge Level: " + chargeLevel.ToString());
            WriteLine("Max Charge Level: " + maxChargeLevel.ToString());

            WriteLine("AF3: " + (int)es.GetContactQuality((int)EdkDll.IEE_InputChannels_t.IEE_CHAN_AF3));
            WriteLine("T7:  " + (int)es.GetContactQuality((int)EdkDll.IEE_InputChannels_t.IEE_CHAN_T7));
            WriteLine("Pz : " + (int)es.GetContactQuality((int)EdkDll.IEE_InputChannels_t.IEE_CHAN_O1));
            WriteLine("T8 : " + (int)es.GetContactQuality((int)EdkDll.IEE_InputChannels_t.IEE_CHAN_T8));
            WriteLine("AF4: " + (int)es.GetContactQuality((int)EdkDll.IEE_InputChannels_t.IEE_CHAN_AF4));
        }
        static void GetBrainInterfaceMeasurements()
        {
            EmoEngine engine = EmoEngine.Instance;
            engine.UserAdded += new EmoEngine.UserAddedEventHandler(engine_UserAdded_Event);
            engine.Connect();

            EdkDll.IEE_DataChannel_t[] channelList = new EdkDll.IEE_DataChannel_t[5]
            {
                    EdkDll.IEE_DataChannel_t.IED_AF3,
                    EdkDll.IEE_DataChannel_t.IED_AF4,
                    EdkDll.IEE_DataChannel_t.IED_T7,
                    EdkDll.IEE_DataChannel_t.IED_T8,
                    EdkDll.IEE_DataChannel_t.IED_O1
            };
            
            while (true)
            {
                engine.ProcessEvents(10);
                if (userID < 0) continue;

                if (KeyAvailable) break;

                double[] alpha = new double[1];
                double[] low_beta = new double[1];
                double[] high_beta = new double[1];
                double[] gamma = new double[1];
                double[] theta = new double[1];

                for (int i = 0; i < 5; i++)
                {
                    engine.IEE_GetAverageBandPowers((uint)userID, channelList[i], theta, alpha, low_beta, high_beta, gamma);
                    if (theta[0] > 0)
                    {
                        #region TestOutput
                        //WriteLine("Channel: " + channelList[i]);
                        //WriteLine(
                        //      "Theta:     " + theta[0].ToString().Substring(0, 5) +
                        //      ", Alpha:     " + alpha[0].ToString().Substring(0, 5) +
                        //      ", Low Beta:  " + low_beta[0].ToString().Substring(0, 5) +
                        //      ", High Beta: " + high_beta[0].ToString().Substring(0, 5) +
                        //      ", Gamma:     " + gamma[0].ToString().Substring(0, 5));
                        #endregion
                        SendBrainMeasurementToAzureAsync(channelList[i].ToString(),
                             theta[0].ToString().Substring(0, 7), alpha[0].ToString().Substring(0, 7),
                            low_beta[0].ToString().Substring(0, 7), high_beta[0].ToString().Substring(0, 7),
                            gamma[0].ToString().Substring(0, 7));
                    }
                }
            }
            engine.Disconnect();
        }
        static void engine_UserAdded_Event(object sender, EmoEngineEventArgs e)
        {
            WriteLine("User Added Event has occured");
            userID = (int)e.userId;
            WriteLine("UserID = " + userID.ToString());
            EmoEngine.Instance.IEE_FFTSetWindowingType((uint)userID, EdkDll.IEE_WindowingTypes.IEE_HAMMING);
        }
        private static async void SendBrainMeasurementToAzureAsync(string channel, string theta, string alpha, string lowbeta, string highbeta, string gamma)
        {
            int ManufacturerId = 1;                         //ex: Emotiv, Inc
            int HardwareId = 1;                             //ex: Emotive Insight 
            int ActivityId = _ActivityId;                   //ex: Bsaeline, Reading, Writing
            int ChannelId = GetChannelId(channel);          //ex: AF3, AF4, Tz, etc...
            int DeviceId = _DeviceId;                       //ex: one of the devices bound to a user name
            string UserName = _UserName;
            DateTime MeasurementDateTime = DateTime.Now;

            var brainActivity = new
            {
                ManufacturerId,
                HardwareId,
                ActivityId,
                ChannelId,
                DeviceId,
                UserName,
                MeasurementDateTime,
                theta,
                alpha,
                lowbeta,
                highbeta,
                gamma
            };
            var messageString = JsonConvert.SerializeObject(brainActivity);
            var message = new Message(Encoding.ASCII.GetBytes(messageString));
            await deviceClient.SendEventAsync(message);  //Make sure you have internet connection
            WriteLine("{0} > Sending message: {1}", DateTime.Now, messageString);
        }
        private static int GetChannelId(string channel)
        {
            switch (channel)
            {
                case "IED_AF3":
                    { return 1; }
                case "IED_AF4":
                    { return 2; }
                case "IED_T7":
                    { return 3; }
                case "IED_T8":
                    { return 4; }
                case "IED_O1":
                    { return 5; }
            }

            return 999;
        }
    }
}
